﻿using System;
using System.Collections.Specialized;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using EPiServer.Core;
using log4net;
using Mediachase.Commerce.Core;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Orders.Dto;
using Mediachase.Commerce.Orders.Managers;
using Mediachase.Commerce.Website;
using Mediachase.Commerce.Website.Helpers;
using EPiServer.Business.Commerce.Payment.PayPal.Orders;
using EPiServer.Framework.DataAnnotations;
using EPiServer.Business.Commerce.Payment.PayPal.PageTypes;

namespace EPiServer.Business.Commerce.Payment.PayPal
{
    [TemplateDescriptor(Path = "~/Templates/Sample/Pages/PayPal.aspx")]
    public partial class PayPal : EPiServer.TemplatePage<PayPalPage>
    {
        public const string PayPalSystemName = "PayPal";
        private static ILog _logger = LogManager.GetLogger(typeof(PayPal));
        private Cart _currentCart = null;

        private Cart CurrentCart
        {
            get
            {
                if (_currentCart == null)
                {
                    _currentCart = new CartHelper(Cart.DefaultName).Cart;
                }

                return _currentCart;
            }
        }

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            // Redirect customer to receipt page
            string acceptUrl = "";
            if (!Request.IsAuthenticated)
            {
                acceptUrl = "~/Templates/Sample/Pages/CheckoutConfirmationStep.aspx";
            }
            else
            {
                var referencePage = DataFactory.Instance.GetPage(PageReference.StartPage)["PaypalPaymentLandingPage"] as PageReference;
                var landingPage = DataFactory.Instance.GetPage(referencePage);
                acceptUrl = landingPage != null ? landingPage.LinkURL : string.Empty;
            }

            string cancelUrl = Utilities.GetUrlValueFromStartPage("CheckoutPage") ?? string.Empty;  // get link to ShoppingCart
            cancelUrl += "#state=step|2&success=false&paymentmethod=paypal";

            string orderID = string.Empty;
            if (CurrentCart.OrderForms.Count == 0 || CurrentCart.OrderForms[0].Payments.Count == 0)
            {
                return;
            }
            var payments = CurrentCart.OrderForms[0].Payments.ToArray();
            //get PayPal payment by PaymentMethodName, instead of get the first payment in the list.
            PaymentMethodDto payPalPaymentMethod = PaymentManager.GetPaymentMethodBySystemName(PayPalSystemName, SiteContext.Current.LanguageName);
            var payment = payments.Where(c => c.PaymentMethodId.Equals(payPalPaymentMethod.PaymentMethod.Rows[0]["PaymentMethodId"])).FirstOrDefault();
            var payPalPayment = payment as PayPalPayment;
            if (payPalPayment == null)
            {
                return;
            }
            orderID = payPalPayment.PayPalOrderNumber;

            if(!string.IsNullOrEmpty(Request.QueryString["hash"]) && string.Equals(Request.QueryString["accept"], "true"))
            {
                PayPalPaymentGateway gateway = new PayPalPaymentGateway();
                if (!string.IsNullOrEmpty(orderID) && PayPalPaymentGateway.GetMD5Key(orderID + "accepted") == Request.QueryString["hash"])
                {
                    gateway.ProcessSuccessfulTransaction(acceptUrl, cancelUrl, payPalPayment);
                }
            }
            else if (!string.IsNullOrEmpty(Request.QueryString["hash"]) && string.Equals(Request.QueryString["accept"], "false"))
            {
                PayPalPaymentGateway gateway = new PayPalPaymentGateway();
                if (!string.IsNullOrEmpty(orderID) && PayPalPaymentGateway.GetMD5Key(orderID + "canceled") == Request.QueryString["hash"])
                {
                    gateway.ProcessUnsuccessfulTransaction(cancelUrl, Utilities.Translate("PayPalCancelMessage"));
                }
            }
        }
    }
}
